/**
 * JASMINe
 * Copyright (C) 2008-2011 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: QuickVisuService.java 9190 2011-10-14 14:37:43Z durieuxp $
 * --------------------------------------------------------------------------
 */

package org.ow2.jasmine.monitoring.eos.quickvisu.service;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;


import org.apache.felix.ipojo.annotations.Component;
import org.apache.felix.ipojo.annotations.Instantiate;
import org.apache.felix.ipojo.annotations.Provides;
import org.apache.felix.ipojo.annotations.ServiceProperty;

import org.ow2.kerneos.core.service.KerneosSimpleService;

import org.ow2.jasmine.monitoring.eos.common.JasmineEventManager;
import org.ow2.jasmine.monitoring.eos.common.jms.JMSConnection;
import org.ow2.jasmine.monitoring.eos.common.jms.event.JasmineMessageEvent;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

@Component
@Instantiate
@Provides
public class QuickVisuService implements KerneosSimpleService {

    protected Log logger = LogFactory.getLog(this.getClass());

    @ServiceProperty(name = KerneosSimpleService.ID, value = "quickvisuService")
    private String id;

    /**
     * serialVersionUID.
     */
    private static final long serialVersionUID = -1254916345191626735L;

    /**
     * JMS topic used for synchronization.
     */
    private static final String JMS_TOPIC = "jasmine";

    /**
     * The JMS connection instance.
     */
    private JMSConnection connection = null;

    /**
     * The event manager instance.
     */
    private JasmineEventManager manager = null;

    /**
     * A Timer to prevent from disconnection.
     */
    private transient Timer timer = null;

    // 2 minutes
    private static long TIMER = 120000;

    public QuickVisuService() {
        this.manager = new JasmineEventManager();
    }

    /**
     * connect to the JMS topic.
     *
     * @return -1 if an error occured, 0 otherwise.
     */
    public void connectTopic() throws Exception {
        logger.debug("");
        if (this.connection == null) {
            this.connection = new JMSConnection(this.manager);
        }

        // set the timer
        this.timer = new Timer();
        this.timer.schedule(new DisconnectTask(), QuickVisuService.TIMER);

        this.connection.connect(QuickVisuService.JMS_TOPIC);
    }

    /**
     * Disconnect from the JMS topic.
     *
     * @return -1 if an error occurred, 0 otherwise.
     */
    public void disconnectTopic() throws Exception {
        logger.debug("");

        if (this.connection != null) {
            this.connection.disconnect();
            this.connection = null;
        }

        if (this.timer != null) {
            this.timer.cancel();
            this.timer = null;
        }
    }

    /**
     * Get all the messages in the queue.
     *
     * @return a list of events.
     */
    public List<JasmineMessageEvent> getMessage() {
        logger.debug("");

        this.resetTimer();

        List<JasmineMessageEvent> list = new ArrayList<JasmineMessageEvent>();

        list.addAll(this.manager.getAll());

        return list;
    }

    /**
     * Reset the timer.
     */
    private synchronized void resetTimer() {
        if (this.timer != null) {
            this.timer.cancel();
        }
        this.timer = new Timer();
        this.timer.schedule(new DisconnectTask(), QuickVisuService.TIMER);
    }

    /**
     * Implementation of TimerTask.
     *
     * @author Guillaume Renault
     */
    class DisconnectTask extends TimerTask {

        @Override
        public void run() {
            try {
                disconnectTopic();
            } catch (Exception e) {
                logger.error("Failed disconnecting from JMS topic", e);
            }
        }

    }

    @SuppressWarnings("unused")
    private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
        // deserialize everything except for the transient fields
        in.defaultReadObject();

        // relaunch JMS timer
        this.resetTimer();
    }

}
